unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ColorBox, Spin, ExtCtrls,
  BGRABitmap, BGRABitmapTypes, BGRATransform, fgl;

type

  { TPlaneItem }

  TPlaneItem = class
    PlaneStream: TMemoryStream;
    Location: TPoint;
    Angle: Integer;
    constructor Create;
    destructor Destroy; override;
  end;

  { TPlaneList }

  TPlaneList = class(specialize TFPGObjectList<TPlaneItem>)
    procedure Fly;
  end;

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    ColorBox1: TColorBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    PaintBox1: TPaintBox;
    SpinEdit1: TSpinEdit;
    SpinEdit2: TSpinEdit;
    FlyTimer: TTimer;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FlyTimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
  private

  public
    function PreparePlane(aColor: TColor; aAngle, aSize: Integer): TMemoryStream;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TPlaneItem }

constructor TPlaneItem.Create;
begin
  inherited Create;
  PlaneStream := nil;
  Location := Point(0, 0);
end;

destructor TPlaneItem.Destroy;
begin
  if Assigned(PlaneStream) then
    PlaneStream.Free;
  inherited Destroy;
end;

{ TPlaneList }

procedure TPlaneList.Fly;
const
  directionarr: Array [0..7, 0..1] of Integer = ((0, -1), (1, -1), (1, 0), (1, 1), (0, 1), (-1, 1), (-1, 0), (-1, -1));
  distance: Integer = 3;
var
  i, direction: Integer;
begin
  for i := 0 to Count - 1 do
  begin
    direction := Items[i].Angle div 45;
    Items[i].Location.X := Items[i].Location.X + trunc(distance * directionarr[direction][0] * (1 + ((Items[i].Angle mod 45) / 45)));
    Items[i].Location.Y := Items[i].Location.Y + trunc(distance * directionarr[direction][1] * (1 + ((Items[i].Angle mod 45) / 45)));
  end;
end;

{ TForm1 }

var
  planelist: TPlaneList;

procedure TForm1.Button1Click(Sender: TObject);
var
  pi: TPlaneItem;
begin
  pi := TPlaneItem.Create;
  pi.PlaneStream := PreparePlane(ColorBox1.Selected, SpinEdit1.Value, SpinEdit2.Value);
  pi.Location.X := Random(PaintBox1.Width - SpinEdit2.Value);
  pi.Location.Y := Random(PaintBox1.Height - SpinEdit2.Value);
  pi.Angle := SpinEdit1.Value;
  planelist.Add(pi);
  PaintBox1.Invalidate;
  Caption := IntToStr(planelist.Count);
  //random color and angle
  ColorBox1.Selected := ColorBox1.Colors[Random(16)];
  SpinEdit1.Value := Random(36) * 10;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  FlyTimer.Enabled := not FlyTimer.Enabled;
  if FlyTimer.Enabled then
    Button2.Caption := 'STOP'
  else
    Button2.Caption := 'Fly';
end;

procedure TForm1.FlyTimerTimer(Sender: TObject);
begin
  planelist.Fly;
  PaintBox1.Invalidate;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Randomize;
  planelist := TPlaneList.Create;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  planelist.Free;
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
var
  i: Integer;
  pi: TPlaneItem;
  bmp, pbmp: TBGRABitmap;
begin
  //draw map
  bmp := TBGRABitmap.Create('OpenStreetMap.jpg');
  //draw all planes
  for i := 0 to planelist.Count - 1 do
  begin
    pi := planelist.Items[i];
    pi.PlaneStream.Position := 0;
    pbmp := TBGRABitmap.Create(pi.PlaneStream);
    bmp.PutImage(pi.Location.X, pi.Location.Y, pbmp, dmDrawWithTransparency);
    pbmp.Free;
  end;
  bmp.Draw(PaintBox1.Canvas, 0, 0);
  bmp.Free;
end;

function TForm1.PreparePlane(aColor: TColor; aAngle, aSize: Integer): TMemoryStream;
var
  bmp, temp: TBGRABitmap;
  bmptranform: TBGRAAffineBitmapTransform;
  w, h: Integer;
begin
  Result := TMemoryStream.Create;
  bmp := TBGRABitmap.Create('flieger.bmp');
  bmp.ReplaceColor(bmp.ClipRect, BGRABlack, BGRAPixelTransparent);  //Replace black to transparent
  bmp.ReplaceColor(clWhite, aColor);    //change plane color
  w := bmp.Width;
  if bmp.Height > w then
    w := bmp.Height;
  //rotate plane
  temp := TBGRABitmap.Create(w, w, BGRAPixelTransparent);
  bmptranform := TBGRAAffineBitmapTransform.Create(bmp);
  bmptranform.Translate(-bmp.Width div 2, -bmp.Height div 2);
  bmptranform.RotateDeg(aAngle);
  bmptranform.Translate(bmp.Width div 2, bmp.Height div 2);
  temp.Fill(bmptranform);
  bmptranform.Free;
  bmp.Assign(temp);
  temp.Free;
  //resize plane
  if (bmp.Width > aSize) or (bmp.Height > aSize) then
  begin
    w := aSize;
    h := aSize;
    if bmp.Width >= bmp.Height then
      h := trunc(bmp.Height * (w / bmp.Width))
    else
      w := trunc(bmp.Width * (h / bmp.Height));
    BGRAReplace(bmp, bmp.Resample(w, h, rmFineResample));
  end;
  bmp.SaveToStreamAsPng(Result);
  bmp.Free;
end;

end.

